#ifndef ECSERIALIZER_H
#define ECSERIALIZER_H

#include "zcdefs.h"
#include <ZarafaCode.h>

struct IStream;
class ECFifoBuffer;

#ifdef DEBUG
#define STR_DEF_TIMEOUT 0
#else
#define STR_DEF_TIMEOUT 600000
#endif

class ECSerializer
{
public:
	virtual ~ECSerializer() {};

	virtual ECRESULT SetBuffer(void *lpBuffer) = 0;

	virtual ECRESULT Write(const void *ptr, size_t size, size_t nmemb) = 0;
	virtual ECRESULT Read(void *ptr, size_t size, size_t nmemb) = 0;

	virtual ECRESULT Skip(size_t size, size_t nmemb) = 0;
	virtual ECRESULT Flush() = 0;
	
	virtual ECRESULT Stat(ULONG *lpulRead, ULONG *lpulWritten) = 0;
};

class ECStreamSerializer _final : public ECSerializer
{
public:
	ECStreamSerializer(IStream *lpBuffer);
	ECRESULT SetBuffer(void *lpBuffer) _override;
	ECRESULT Write(const void *ptr, size_t size, size_t nmemb) _override;
	ECRESULT Read(void *ptr, size_t size, size_t nmemb) _override;
	ECRESULT Skip(size_t size, size_t nmemb) _override;
	ECRESULT Flush(void) _override;
	ECRESULT Stat(ULONG *lpulRead, ULONG *lpulWritten) _override;

private:
	IStream *m_lpBuffer;
	ULONG m_ulRead;
	ULONG m_ulWritten;
};

class ECFifoSerializer _final : public ECSerializer
{
public:
	enum eMode { serialize, deserialize };

	ECFifoSerializer(ECFifoBuffer *lpBuffer, eMode mode);
	virtual ~ECFifoSerializer(void);
	ECRESULT SetBuffer(void *lpBuffer) _override;
	ECRESULT Write(const void *ptr, size_t size, size_t nmemb) _override;
	ECRESULT Read(void *ptr, size_t size, size_t nmemb) _override;
	ECRESULT Skip(size_t size, size_t nmemb) _override;
	ECRESULT Flush(void) _override;
	ECRESULT Stat(ULONG *lpulRead, ULONG *lpulWritten) _override;

private:
	ECFifoBuffer *m_lpBuffer;
	eMode m_mode;
	ULONG m_ulRead;
	ULONG m_ulWritten;
};

#endif /* ECSERIALIZER_H */
